// PWPFILTER.CPP - Implementation file for your Internet Server
//    Personal Web Pages service Isapi Filter

/*
	This Isapi filter will take an incoming url of 
	http://www.mypwpsite.com/~<userName> and redirect it to
	http://www.myPwpSite.com/pwpPublish/<hashNumber>/<userName>
	while at the same time leaving the url displayed in the users
	address box	untouched.  This is done to  support users not 
	having to use and/or see their actual hash valued folder paths
	when attempting to connected to their pwp home pages.
*/

/*
	Dec98 - Dennis Mabrey built original url redirection filter.
	Feb99 - Robert O'Brien migrated to C++ version and updated to
	use the COM	accessible version of the icuUtils CalcHashValue
	routine	versus the older api version.
	May99 - Ronald Rosenberg resolved some bugs that were found, 
	added the event logging routines and got the OnUrlMap method
	working that had been previously shuffled out to the readMe.txt
	due to bugs.
*/

#include "stdafx.h"
#include "pwpFilter.h"

// additions
// use #import statement to compile once and generate the type
// library header file (.tlh) specified and then switch to using
// the static header file versus the import directive
// #import "c:\program files\common files\icu\icuUtils.dll" no_namespace
#include "icuUtils.tlh"
#pragma comment(lib, "mpr.lib")  // used by the WNetCancelConnection2 calls


///////////////////////////////////////////////////////////////////////
// The one and only CWinApp object
// NOTE: You may remove this object if you alter your project to no
// longer use MFC in a DLL.

CWinApp theApp;


///////////////////////////////////////////////////////////////////////
// The one and only CPwpFilter object

CPwpFilter theFilter;


///////////////////////////////////////////////////////////////////////
// CPwpFilter implementation

CPwpFilter::CPwpFilter()
{
	#ifdef _DEBUG
		DebugBreak(); // used to force debugger break point at this point
	//	__asm int 3  // in-line assembler approach
	#endif
	/*RAR*/
	// Set service name for logging:
	wsprintf(m_szServiceName, (LPCTSTR)"pwpFilter");
	// Registry path in HKLM:
	_TCHAR szRegistryHome[] = _T("Software\\Network Solutions Group\\Pwp");
	// Temp string for logging:
	_TCHAR szT[128];

	// zero and then process global member assignments in constructor
	HRESULT hr = E_FAIL; LONG lr = 0; DWORD dwr = 0;
	m_bRegDataOk = false; m_bUseCentralReg = false; 
	_tcsnset(m_szCentralRegHost, _T('\0'), MAX_BUFF);
	m_lHashMax = 0;
	_tcsnset(m_szWebVdirPublish, _T('\0'), MAX_BUFF);
	
	// initialize COM
	// not required since as this is already done
	// as part of the isapi environment in the mta
	// hr = CoInitialize(0);  // use Ex version for more control
	// hr = CoInitializeEx(0, COINIT_MULTITHREADED);	
	
	HKEY hKey = HKEY_LOCAL_MACHINE, hSubKey = 0;
	DWORD cbValue = MAX_BUFF, dwType = 0;
	
	// first look up hostname to see if a centralized
	// configuration registry has been defined for use
    lr = RegOpenKeyEx(hKey, szRegistryHome, 0,
					  KEY_ALL_ACCESS, &hSubKey);  // get hSubKey
	if (lr != ERROR_SUCCESS )
	{
		// can't open registry, we're toast:
		_stprintf(szT, "pwpFilter failed to open the local machine registry using %s",
			szRegistryHome );
		LogEvent( EVENTLOG_ERROR_TYPE, 0, szT);
		return;
	}
	else
	{
		cbValue = MAX_BUFF;
		lr = RegQueryValueEx(hSubKey, _T("CentralRegHost"), 0,
							 &dwType, (LPBYTE)m_szCentralRegHost, &cbValue);
		m_bRegDataOk = true;
		// now if necessary connect to centralized registry configuration host
		if (0 == _tcsicmp(_T("\0"), m_szCentralRegHost))  // null registry hostname means use localhost registry
		{
			m_bUseCentralReg = false;
		}
		else
		{
			m_bUseCentralReg = true;
			// Close local registry:
			RegCloseKey(hSubKey);
			lr = RegConnectRegistry(m_szCentralRegHost, hKey, &hKey);  // get remoted hKey
			if (lr != ERROR_SUCCESS)
			{
				// unable to connect to remote registry
				_stprintf(szT, "unable to connect to remote registry at %s",
					m_szCentralRegHost );
				LogEvent( EVENTLOG_ERROR_TYPE, 0, szT);
				dwr = WNetCancelConnection2(_tcscat(m_szCentralRegHost, _T("\\ipc$")), 0, false);
				return;
			}
		}
	}
	
	if (m_bRegDataOk) // only if we haven't errored out yet on registry lookups
	{
		// acquire the hash maximum from registry
		lr = RegOpenKeyEx(hKey, szRegistryHome, 0,
						  KEY_ALL_ACCESS, &hSubKey);  // get hSubKey
		if (lr)
			m_bRegDataOk = false;
		else
		{
			#ifdef _DEBUG
				DebugBreak(); // used to force debugger break point at this point
			#endif
			cbValue = sizeof(DWORD);
			lr = RegQueryValueEx(hSubKey, _T("HashMax"), 0, 
								 &dwType, (LPBYTE)&m_lHashMax, &cbValue);
			m_bRegDataOk = true;
		}
		// If we can't get the hash max we're toast:
		if (m_lHashMax <= 0 )
		{
			LogEvent( EVENTLOG_ERROR_TYPE, 0, "pwpFilter failed to get the hash max");
			RegCloseKey(hKey);
			RegCloseKey(hSubKey);
			return;
		}
	}

    if (m_bRegDataOk) // only if we haven't errored out yet on registry lookups
	{
		// acquire the web virtual directory path for publishing from registry
		cbValue = MAX_BUFF;
		lr = RegQueryValueEx(hSubKey, _T("WEBVDIR_PUBLISH"), 0, 
							 &dwType, (LPBYTE)m_szWebVdirPublish, &cbValue);
		m_bRegDataOk = true;
	}

	// now close down registry handles
	RegCloseKey(hSubKey);
	RegCloseKey(hKey);
	if (m_bUseCentralReg)
		dwr = WNetCancelConnection2(_tcscat(m_szCentralRegHost, _T("\\ipc$")), 0, false);

	// no returns required | supported in constructor
}


CPwpFilter::~CPwpFilter()
{
	#ifdef _DEBUG
	//	DebugBreak();  // used to force debugger break point at this point
	//	__asm int 3  // in-line assembler approach
	#endif

	// do global member variable deassignments in deconstructor
	m_lHashMax = 0;
	_tcsnset(m_szWebVdirPublish, _T('\0'), MAX_BUFF);
	
	// uninitialize COM
	// note - not required since as this is already done
	// as part of the isapi environment in the mta
	// CoUninitialize();

	// no returns required | supported in constructor
}

BOOL CPwpFilter::GetFilterVersion(PHTTP_FILTER_VERSION pVer)
{
	#ifdef _DEBUG
	//	DebugBreak();  // used to force debugger break point at this point
	//	__asm int 3  // in-line assembler approach
	#endif
	
	// If we couldn't get the hash max we're toast so return 0 so we don't get loaded:
	if (m_lHashMax <= 0)
	{
		LogEvent(EVENTLOG_ERROR_TYPE, 0, "pwpFilter failing due to bad hash max, and won't get loaded");
		return 0;
	}

	// Call default implementation for initialization
	CHttpFilter::GetFilterVersion(pVer);

	// Clear the flags set by base class
	pVer->dwFlags &= ~SF_NOTIFY_ORDER_MASK;

	// Set the flags we are interested in
//	pVer->dwFlags |= SF_NOTIFY_ORDER_LOW | SF_NOTIFY_SECURE_PORT | SF_NOTIFY_NONSECURE_PORT
//			 | SF_NOTIFY_PREPROC_HEADERS;
	pVer->dwFlags |= SF_NOTIFY_ORDER_LOW | SF_NOTIFY_SECURE_PORT | SF_NOTIFY_NONSECURE_PORT
			 | SF_NOTIFY_URL_MAP;

	// Load description string
	TCHAR sz[SF_MAX_FILTER_DESC_LEN+1];
	ISAPIVERIFY(::LoadString(AfxGetResourceHandle(),
			IDS_FILTER, sz, SF_MAX_FILTER_DESC_LEN));
	_tcscpy(pVer->lpszFilterDesc, sz);
	return TRUE;
}


DWORD CPwpFilter::OnUrlMap(CHttpFilterContext* pCtxt, PHTTP_FILTER_URL_MAP pUrlMap) 
{
	#ifdef _DEBUG
		DebugBreak();
	#endif
	
	HRESULT hr = E_FAIL;
	CString sPhysPath, sTokenStr;
	_TCHAR szNewTokenStr[MAX_BUFF] = {_T("\0")};
	long lHashValue = 0;
	int nStartTokenStr = 0, nEndTokenStr = 0, nLenTokenStr = 0;
		
	sPhysPath = pUrlMap->pszPhysicalPath;
	if (0 == sPhysPath.GetLength()) return CHttpFilter::OnUrlMap(pCtxt, pUrlMap);

	// check if this is a physical path we want to
	// remap by looking for the "~" token in the path
	nStartTokenStr = sPhysPath.FindOneOf("~");
	if (0 < nStartTokenStr)  // we found a token in the path
	{	
		nEndTokenStr = sPhysPath.Mid(nStartTokenStr).FindOneOf(_T("\\"));
		if (0 < nEndTokenStr)  // in case path is terminated with '\' and or destination file name
		{
			sTokenStr = sPhysPath.Mid(nStartTokenStr, nEndTokenStr);
			nLenTokenStr = nEndTokenStr - nStartTokenStr;
		}
		else
		{
			sTokenStr = sPhysPath.Mid(nStartTokenStr);
			nLenTokenStr = sPhysPath.GetLength() - nStartTokenStr;
		}
		// acquire the necessary COM interface smartpointer
		CComPtr<IHashValue> pHashValue;
		hr = pHashValue.CoCreateInstance(__uuidof(HashValue), NULL, CLSCTX_ALL);
		if (SUCCEEDED(hr))
		{
			// calculate the hashvalue of the usernamet string
			lHashValue = pHashValue->CalcHashValue((LPCTSTR)sTokenStr.Mid(1), &m_lHashMax);
			pHashValue.Release();
		}
		else
		{
			// return without modifying the url because
			// we wont be able to calculate the HashValue
			// necessary to generate physical path remap
			return CHttpFilter::OnUrlMap(pCtxt, pUrlMap);
		}
					
		// build new redirected physical path for url token string
		wsprintf(szNewTokenStr, _T("%ld\\%s"), lHashValue, (LPCTSTR)sTokenStr.Mid(1));
		sPhysPath.Replace((LPCTSTR)sTokenStr, szNewTokenStr);
		
		// place new redirected path back in url map path member and reset buffer length
		_stprintf(pUrlMap->pszPhysicalPath, "%s", sPhysPath);
	}

	// return control back to iis now that url has been remapped
	return CHttpFilter::OnUrlMap(pCtxt, pUrlMap);
}
DWORD CPwpFilter::OnPreprocHeaders(CHttpFilterContext* pCtxt,
	PHTTP_FILTER_PREPROC_HEADERS pHeaderInfo)
{
	#ifdef _DEBUG
		DebugBreak();  // used to force debugger break point at this point
	//	__asm int 3  // in-line assembler approach
	#endif
	
	HRESULT hr = E_FAIL;
	_TCHAR szUrl[MAX_URL] = {_T("\0")},
		szNewUrl[MAX_URL] = {_T("\0")},
		  aBuffer[MAX_BUFF] = {_T("\0")};
	DWORD dwBuffSize = MAX_URL;
	long lHashValue = 0;
	int nIndex = 0; 
	
	
	hr = pHeaderInfo->GetHeader(pCtxt->m_pFC, _T("url"), (void *)szUrl, &dwBuffSize);
	if (FAILED(hr)) return SF_STATUS_REQ_NEXT_NOTIFICATION;

	
	if (szUrl[0] == '/' && szUrl[1] == TOKEN)  // check if this is an url we care about
	{	
		// extract the username from the url
		nIndex = 2;
		while (szUrl[nIndex] != '\0'  && szUrl[nIndex] != '/')
		{
			aBuffer[nIndex - 2] = szUrl[nIndex];
			nIndex++;
		}
		aBuffer[nIndex - 2] = '\0';  // ensure buffer is null terminated

		#ifdef _DEBUG
			DebugBreak();
		#endif
		
		// acquire the necessary COM interface smartpointer
		CComPtr<IHashValue> pHashValue;
		hr = pHashValue.CoCreateInstance(__uuidof(HashValue), NULL, CLSCTX_ALL);
		if (SUCCEEDED(hr))
		{
			// calculate the hashvalue of the usernamet string
			lHashValue = pHashValue->CalcHashValue(aBuffer, &m_lHashMax);
			pHashValue.Release();
		}
		else
		{
			// return without modifying the url because
			// we wont be able to calculate the HashValue
			// necessary to generate url remap
			return SF_STATUS_REQ_NEXT_NOTIFICATION;
		}
					
		// build new redirected url string
		if (0 == _tcsicmp(m_szWebVdirPublish, _T("\0")))  // check if m_szWebVdirPublish specifies pwp web root
			wsprintf(szNewUrl, _T("/%ld/%s"), lHashValue, szUrl + 2);
		else  // m_sWebVdirPublish does not specifiy the pwp web root
			wsprintf(szNewUrl, _T("/%s/%ld/%s"), m_szWebVdirPublish, lHashValue, szUrl + 2);

		// place new redirected url back in request header
		pHeaderInfo->SetHeader(pCtxt->m_pFC, "url", szNewUrl);
	}

	// return control back to iis now that url has been remapped
	return SF_STATUS_REQ_NEXT_NOTIFICATION;
}


// Do not edit the following lines, which are needed by ClassWizard.
#if 0
BEGIN_MESSAGE_MAP(CPwpFilter, CHttpFilter)
	//{{AFX_MSG_MAP(CPwpFilter)
	//}}AFX_MSG_MAP
END_MESSAGE_MAP()
#endif	// 0


///////////////////////////////////////////////////////////////////////
// If your extension will not use MFC, you'll need this code to make
// sure the extension objects can find the resource handle for the
// module.  If you convert your extension to not be dependent on MFC,
// remove the comments around the following AfxGetResourceHandle()
// and DllMain() functions, as well as the g_hInstance global.

/****

static HINSTANCE g_hInstance;

HINSTANCE AFXISAPI AfxGetResourceHandle()
{
	return g_hInstance;
}

BOOL WINAPI DllMain(HINSTANCE hInst, ULONG ulReason,
					LPVOID lpReserved)
{
	if (ulReason == DLL_PROCESS_ATTACH)
	{
		g_hInstance = hInst;
	}

	return TRUE;
}

****/

BOOL CPwpFilter::LogEvent(short wType, long dwID, LPCTSTR Message) 
{
	// The error types are:
	// EVENTLOG_ERROR_TYPE, EVENTLOG_WARNING_TYPE, EVENTLOG_INFORMATION_TYPE
	//
    // Check to see if the event source has been registered and if
    // not then register it now
    if (!m_hEventSource) {
        m_hEventSource = ::RegisterEventSource(NULL,  // local machine
                                               m_szServiceName); // source name
    }

    if (m_hEventSource) {
        ::ReportEvent(m_hEventSource,
                      wType,
                      0,
                      dwID,
                      NULL, // sid
                      1,
                      0,
                      &Message,
                      NULL);
    }
	else {
		return FALSE;
	}

	return TRUE;
}
